#include "bmp.hpp"

struct BitmapFileHeader
{
	char Magic[2];
	unsigned int FileSize;
	unsigned int tmp;
	unsigned int ImageOffset;
} __attribute__ ((packed));
struct BitmapInfoHeader
{
	unsigned int InfoHeaderSize;
	int Width;
	int Height;
	unsigned short Planes;
	unsigned short BitSize;
	unsigned int Compress;
	unsigned int HDpi;
	unsigned int VDpi;
	unsigned int PalUsed;
	unsigned int PalImportant;
} __attribute__ ((packed));

bool 
Bitmap::load(std::string filename)
{
	int now_size = bmp.size();
	if (isLoaded(filename)) {
		return getIndex(filename);
	}

	FILE *file;
	if ((file = fopen(filename.c_str(), "rb")) == NULL) {
		fprintf(stderr, "Can't open file %s\n", filename.c_str());
		return false;
	}
	fseek(file, 0, SEEK_END);
	int size = ftell(file);
	fseek(file, 0, SEEK_SET);

	char* buf = new char[size];
	fread(buf, 1, size, file);
	fclose(file);

	bmp.push_back(buf);
	if (now_size != bmp.size()) {
		name.insert(std::pair<std::string, int>(std::string(filename), now_size));
	}
	return true;
}

int 
Bitmap::getIndex(std::string filename)
{
	std::map<std::string, int>::iterator it = name.find(filename);
	if (it == name.end())
		return -1;
	else
		return it->second;
}

int 
Bitmap::getWidth(std::string filename)
{
	int index = getIndex(filename);
	if (index < 0) return 0;
	BitmapFileHeader *bh = (BitmapFileHeader*)(bmp[index]);
	BitmapInfoHeader *bi = (BitmapInfoHeader*)(&bh[1]);
	
	return bi->Width;
}

int 
Bitmap::getHeight(std::string filename)
{
	int index = getIndex(filename);
	if (index < 0) return 0;
	BitmapFileHeader *bh = (BitmapFileHeader*)(bmp[index]);
	BitmapInfoHeader *bi = (BitmapInfoHeader*)(&bh[1]);
	
	return bi->Height;
}

bool 
Bitmap::print8x8(std::string& str, std::string filename, int x, int y) {
	int index = getIndex(filename);
	if (index < 0) return false;

	BitmapFileHeader *bh = (BitmapFileHeader*)(bmp[index]);
	BitmapInfoHeader *bi = (BitmapInfoHeader*)(&bh[1]);

	char* b = bmp[index] + bh->ImageOffset;
	char buf[64];

	str += "	.short ";

	for (int yy = 0; yy < 8; yy++) {
		unsigned int pal = 0;
		char* bx = &b[(bi->Height - (y + yy) - 1) * bi->Width + x];
		for (int xx = 0; xx <= 1; ++xx) {
			if (yy == 0 && xx == 0)
				sprintf(buf, "0x%X%X%X%X", bx[3]&15, bx[2]&15, bx[1]&15, bx[0]&15);
			else 
				sprintf(buf, ",0x%X%X%X%X", bx[3]&15, bx[2]&15, bx[1]&15, bx[0]&15);
			str += buf;
			bx +=4;
		}
	}
	str += "\n";
	return true;
}
