/*
 *  Converts a bmp file into an c array
 */
#include <iostream>
#include <vector>
#include <string>
#include "bmp.hpp"
#include "parse.hpp"

void parse_opt(char *s);
FILE *cout;
FILE *hout;

#define	__USAGE__	\
"Usage: bmp2c [options] input \n"

static bool debug_print = false;

Bitmap bmp;
void get_basename(std::string &str, const char *c);

//=============================================================================
struct CStock
{
	struct Stock {
		std::string bmp_name;
		std::string obj_name;
		int x, y;
		int x_size, y_size;
		int size_type;
	};
	std::string now_bmp;

	std::vector<Stock> stockArray;
	Stock tmp_stock;
	Stock tmp_stock_old;
	CStock() {
		now_bmp = "";
		tmp_stock.bmp_name = "";
		tmp_stock.obj_name = "";
		tmp_stock.x = -1;
		tmp_stock.y = -1;
		tmp_stock.x_size = 8;
		tmp_stock.y_size = 8;
		tmp_stock.size_type = 0;
	}
	~CStock() {}

	void 
	initTempStock(void)
	{
		tmp_stock.bmp_name = "";
		tmp_stock.obj_name = "";
		tmp_stock.x = -1;
		tmp_stock.y = -1;
		tmp_stock.x_size = -1;
		tmp_stock.y_size = -1;
		tmp_stock.size_type = -1;
	}

	void print(Bitmap *b) {
		std::vector<Stock>::iterator citr = stockArray.begin();
		std::vector<Stock>::iterator cend = stockArray.end();
		for ( ; citr != cend; ++citr) {
			if (!b->isLoaded(citr->bmp_name))
				printf("* ");
			printf("[%s] ",citr->bmp_name.c_str());
			printf("%s : ",citr->obj_name.c_str());
			printf("(%d,%d)", citr->x, citr->y);
			printf(" %dx%d\n", citr->x_size, citr->y_size);
		}
	}
	bool print8x8(Bitmap *b, const std::string& name, const std::string& path) {
		std::vector<Stock>::iterator citr = stockArray.begin();
		std::vector<Stock>::iterator cend = stockArray.end();
		bool ret = true;
		int addr = 0;

		std::string asm_s;
		std::string asm_h;
		std::string enum_h;
		std::string ptn_c;
		std::string ptn_h;
		char buf[256];

		// Assembly
		sprintf(buf, "\t@ %s\n",citr->obj_name.c_str());
		asm_s  = "\t.section\t.rodata\n";
		asm_s += "\t.align\t2\n";
		sprintf(buf, "\t.global\t%s_chr\n\t.type\t%s_chr, object\n%s_chr:\n", name.c_str(), name.c_str(), name.c_str());
		asm_s += buf;
		// Header
		sprintf(buf, "#ifndef %s_CHR_H\n#define %s_CHR_H\n#include <gba.h>\nextern const u16 %s_chr[];\n#endif\n", 
			name.c_str(),name.c_str(),name.c_str());
		asm_h = buf;
		// Enum
		sprintf(buf, "#ifndef %s_ENUM_H\n#define %s_ENUM_H\nenum {\n", 
			name.c_str(),name.c_str());
		enum_h = buf;
		// Pattern
		sprintf(buf, "#include \"%s_pat.h\"\nconst SPR_PAT %s_pat[] = {\n", name.c_str(), name.c_str());
		ptn_c = buf;
		// Pattern Header
		sprintf(buf, "#ifndef %s_PAT_H\n#define %s_PAT_H\n", name.c_str(),name.c_str());
		ptn_h = buf;
		sprintf(buf, "#include <gba.h>\n#include <sprite.h>\n#include \"%s_enum.h\"\n#include \"%s.h\"\nextern const SPR_PAT %s_pat[];\n", name.c_str(), name.c_str(), name.c_str());
		ptn_h += buf;


		for ( ; citr != cend; ++citr) {
			if (b->isLoaded(citr->bmp_name)) {
				sprintf(buf, "\t@ %s\n",citr->obj_name.c_str());
				asm_s += buf;

				sprintf(buf, "\t%s,\n",citr->obj_name.c_str());
				enum_h += buf;

				sprintf(buf, "\t[%-16s] = { (u16*)&%s_chr[%5d], OBJ_SIZE_%dx%d, %d, %d, 0 },\n",
					citr->obj_name.c_str(), 
					name.c_str(), 
					addr,
					citr->x_size, 
					citr->y_size,
					citr->x_size / 2,
					citr->y_size / 2
					);
				ptn_c += buf;
				
				for (int yy = 0; yy < citr->y_size; yy += 8) {
					for (int xx = 0; xx < citr->x_size; xx += 8) {
						b->print8x8(asm_s, citr->bmp_name, citr->x + xx, citr->y + yy);
						addr += 16;
					}
				}
			} else {
				fprintf(stderr, "%s not found\n", citr->bmp_name.c_str());
				ret = false;
			}
		}
		asm_s += "\t.ident \"bmp2s\"\n";
		enum_h += "};\n#endif\n";
		ptn_c += "};\n";
		ptn_h += "#endif\n";

		FILE *out;
		sprintf(buf, "%s.s", path.c_str());
		out = fopen(buf, "w");
		fprintf(out, "%s", asm_s.c_str());
		fclose(out);

		sprintf(buf, "%s.h", path.c_str());
		out = fopen(buf, "w");
		fprintf(out, "%s", asm_h.c_str());
		fclose(out);

		sprintf(buf, "%s_enum.h", path.c_str());
		out = fopen(buf, "w");
		fprintf(out, "%s", enum_h.c_str());
		fclose(out);

		sprintf(buf, "%s_pat.c", path.c_str());
		out = fopen(buf, "w");
		fprintf(out, "%s", ptn_c.c_str());
		fclose(out);

		sprintf(buf, "%s_pat.h", path.c_str());
		out = fopen(buf, "w");
		fprintf(out, "%s", ptn_h.c_str());
		fclose(out);
		return ret;
	}
};

//=============================================================================
CStock g_stk;
enum error_return {
	NO_ERR,
	FILE_OPEN_ERROR,
} error = NO_ERR;

//=============================================================================

void 
pushFilename(const char* const a, const char* const b)
{
	int len = b - a;
	std::string str(a, len);
	g_stk.now_bmp = str;
	if (!bmp.load(str))
	{
		error = FILE_OPEN_ERROR;
	}
}
void 
pushObjname(const char* const a, const char* const b)
{
	int len = b - a;
	std::string str(a, len);
	g_stk.tmp_stock.obj_name = str;
}
void 
pushXpos(const int i)
{
	g_stk.tmp_stock.x = i;
}
void 
pushYpos(const int i)
{
	g_stk.tmp_stock.y = i;
}
void 
pushSize(const char* const a, const char* const b)
{
	int len = b - a;
	std::string str(a, len);
	CStock::Stock& t = g_stk.tmp_stock;
	if (str == "8x8") { t.x_size = 8; t.y_size= 8; }
	else if (str == "16x16") { t.x_size = 16; t.y_size= 16; }
	else if (str == "32x32") { t.x_size = 32; t.y_size= 32; }
	else if (str == "64x64") { t.x_size = 64; t.y_size= 64; }
	else if (str == "16x8" ) { t.x_size = 16; t.y_size= 8; }
	else if (str == "32x8" ) { t.x_size = 32; t.y_size= 8; }
	else if (str == "32x16") { t.x_size = 32; t.y_size= 16; }
	else if (str == "64x32") { t.x_size = 64; t.y_size= 32; }
	else if (str == "8x16" ) { t.x_size = 8;  t.y_size= 16; }
	else if (str == "8x32" ) { t.x_size = 8;  t.y_size= 32; }
	else if (str == "16x32") { t.x_size = 16; t.y_size= 32; }
	else if (str == "32x64") { t.x_size = 32; t.y_size= 64; }
}

void 
addStock(const char* const a, const char* const b)
{
	int xsz;
	int ysz;
	CStock::Stock& t = g_stk.tmp_stock;
	CStock::Stock& t_old = g_stk.tmp_stock_old;
	if (t.obj_name == "") {
		g_stk.initTempStock();
		return;
	}
	t.bmp_name = g_stk.now_bmp;
	if (t.y_size == -1) {
		t.x_size = t_old.x_size;
		t.y_size = t_old.y_size;
		t.size_type = t_old.size_type;
	}
	xsz = t_old.x_size;
	ysz = t_old.y_size;
	if (t.y == -1) {
		t.x = t_old.x;
		t.y = t_old.y;
		int h = bmp.getHeight(g_stk.now_bmp);
		int w = bmp.getWidth(g_stk.now_bmp);
		if (t.x + xsz + t.x_size > w) {
			t.x = 0;
			t.x += ysz;
		} else {
			t.x += xsz;
		}
	}

	g_stk.stockArray.push_back(t);
	t_old = t;
	g_stk.initTempStock();
}

//=============================================================================
// 
//=============================================================================
int main(int argc, char **argv)
{
	std::string str;
	char strbuf[512];
	int index;
	for (index = 1; index < argc; index++) {
		if (*argv[index] == '-') 
			parse_opt(argv[index] + 1);
		else 
			break;
	}
	if (index == argc) {
		fprintf(stderr, __USAGE__);
		return 1;
	}

	FILE* fp;
	if ((fp = fopen(argv[index], "r")) == NULL) {
		fprintf(stderr, "Error: Can't open file '%s'\n", argv[index]);
		return 1;
	}
	while(NULL != fgets(strbuf, 512, fp))
	{
		str += strbuf;
	}
	fclose(fp);

	if(!parse_txt(str))
		fprintf(stderr, "Illegal input file\n");

	std::string basename; // only filename
	std::string pathname; // with path
	if (argc > index+1) {
		pathname = argv[index+1];
		get_basename(basename, argv[index+1]);
	} else {
		get_basename(basename, argv[index]);
		pathname = basename;
	}

	if(debug_print) {
		g_stk.print(&bmp);
	} else 
	if (error == NO_ERR) {
		g_stk.print8x8(&bmp, basename, pathname);
	}

	return error;
}

//
void get_basename(std::string &str, const char *c)
{
	char tmp[256];
	char *ps;
	char *pe;
	size_t len;

	ps = strrchr(c, '/');
	if (ps == NULL) 
		ps = strrchr(c, '\\');

	if (ps == NULL) 
		ps = const_cast<char *>(c);
	else 
		ps++;

	pe = strrchr(c, '.');
	if (pe == NULL || pe < ps)
		pe = strrchr(c, 0);

	len = pe - ps;

	strncpy(tmp, ps, pe-ps);
	tmp[len] = '\0';
	str = tmp;
}

//-----------------------------------------------------------------------------
// parse option
//-----------------------------------------------------------------------------
void parse_opt(char *s)
{
	if (*s != '\0') {
		switch (*s) {
		case 'd':
			debug_print = true;
			break;
		default:
			break;
		}
	}
	return;
}
