#include "act.hpp"
#include <boost/spirit/core.hpp>
#include <boost/spirit/utility/confix.hpp>
using namespace boost::spirit;

struct comment_parser : public grammar<comment_parser> {
	template<typename S> struct definition {
		rule<S> comment;
		definition(const comment_parser& self) {
			comment =
				comment_p("//")
				| comment_p("/*", "*/")
				;
		}
		const rule<S>& start() const { return comment; }
	};
};

struct txt_parser : public grammar<txt_parser> 
{
	template<typename S> 
	struct definition 
	{
		rule<S> txt, def, filename, objname, position, size;

		definition(const txt_parser& self)
		{
			filename = '*'
						>> (+(anychar_p - '*' - space_p))[&pushFilename]
						>> +eol_p >> *space_p;
			objname  = ((anychar_p - '*' - space_p - int_p)  >> +(anychar_p - '*' - space_p))[&pushObjname]
						>> *blank_p;
			position = (int_p[&pushXpos]
						>> *blank_p 
						>> ch_p(',') 
						>> *blank_p 
						>> int_p[&pushYpos])
						>> *blank_p;
//			size     =  (int_p >> ch_p('x') >> int_p)[&pushSize];
			size     = (str_p("8x8") 
						| str_p("16x16") 
						| str_p("32x32") 
						| str_p("64x64") 
						| str_p("16x8") 
						| str_p("32x8") 
						| str_p("32x16") 
						| str_p("64x32") 
						| str_p("8x16") 
						| str_p("8x32") 
						| str_p("16x32") 
						| str_p("32x64"))[&pushSize];

			def = filename >> *( objname >> !position >> !size >> *space_p >> eps_p[&addStock]) >> *space_p;
			txt = *space_p >> +def >> *space_p;
		}

		const rule<S>& start() const { return txt; }
	};
};

bool parse_txt(std::string str) {
	txt_parser txt_parse;
	comment_parser comment;
	parse_info<> r = parse(str.c_str(), txt_parse, comment);
	
	if (r.hit)
		return true;
	else
		return false;
}
